/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="ExtTrigInitSearchCV"> ********
Author: Rahkola, R
$Id: ExtTrigInitSearch.c,v 1.6 2002/06/07 20:55:41 rrahkola Exp $
********* </lalVerbatim> ********/

/* Include input/output functions if we're in the debug stage */
#ifdef DEBUG_EXTTRIG
#include <stdio.h>
#endif

/* standard C functions */
#include <math.h>

/* standard LAL functions*/
#include <lal/LALStdlib.h>
#include <lal/Units.h>

/* Associated headers */
#include <ExtTrig.h>
#include <ExtTrigInitSearch.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID( EXTTRIGINITSEARCHC, "$Id: ExtTrigInitSearch.c,v 1.6 2002/06/07 20:55:41 rrahkola Exp $");

/************************* wrapperAPI function definitions  ******************/

/*
 * Function LALInitSearch
 *
 * Initialise the search structures
 */
void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams)
{
  /* allocate memory to const object so we can check our type casts */
  ETSearchParams   *params = NULL;

  /* initialize the lal status structure */
  INITSTATUS( status, "LALInitSearch", EXTTRIGINITSEARCHC );
  ATTATCHSTATUSPTR (status);

#ifdef DEBUG_EXTTRIG
  fprintf(stdout, "exttrig:LALInitSearch\n");
  fflush(stdout);
#endif
  
  /*
   *
   * Sanity checks on the input, output, params
   */
  
  /* these structures should always be correctly created by the wrapperAPI */
  /* so we can use the ASSERT() macro which is disabled in the production  */
  /* code (when lal is compiled with --disable-debug)                      */
  ASSERT(     searchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  ASSERT(  ! *searchParams, status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL );
  ASSERT( initSearchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );

  /* the command line parameters can easily be entered incorrectly at      */
  /* runtime so we should use if() ABORT() test to ensure that errors      */
  /* are always caught even in production code                             */

  if ( !(initSearchParams->argv) ){ /* no filter parameters */
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  }
  if ( initSearchParams->argc != 13) { /* wrong # of filter parameters */
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }
 
  if (strcmp( initSearchParams->argv[0], "-filterparams" )){ /* always first */
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
  }
  if ( atoi( initSearchParams->argv[1] ) <= 650000000 ){ /* not GPS seconds */
    ABORT( status, INITSEARCHH_ENUML, INITSEARCHH_MSGENUML );
  }
  if ( atoi( initSearchParams->argv[2] ) < 0 ){ /* not GPS nanoseconds */
    ABORT( status, INITSEARCHH_ENUML, INITSEARCHH_MSGENUML );
  }
  /* input parameters mask should be even, as 2^0 is reserved for on/off-src */
  if ( atoi( initSearchParams->argv[12] ) & 1 ) {
    ABORT( status, INITSEARCHH_EPAR, INITSEARCHH_MSGEPAR );
  }
  /* check that we have enough data for the number of off-source segments we
   * are asking for
   */
  if ( atoi(initSearchParams->argv[8]) * (atoi(initSearchParams->argv[7]) +1) >
       atoi(initSearchParams->argv[11]) - atoi(initSearchParams->argv[10]) ) {
    ABORT( status, INITSEARCHH_ETOTL, INITSEARCHH_MSGETOTL );
  }

  /* this is the structure that carries memory information between the     */
  /* four functions in the shared object library and between sucessive     */
  /* calls to LALApplySearch()                                             */
  *searchParams = params = LALMalloc( sizeof(ETSearchParams) );
  if ( !params )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }

  /* what follows are the same allocation and checks for the associated    */
  /* substructures -- ETInitSearchParams, ETResults (& substructures)--    */
  /* :NOTE: ETDataSegmentVector and RealFFTPlan get "created" last         */
  params->initParams = (ETInitParams *) LALMalloc ( sizeof(ETInitParams) );
  if ( !(params->initParams) )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }
  params->result = (ETResult *) LALMalloc ( sizeof(ETResult) );
  if ( !(params->result) )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }


  /*
   * assign the initial parameter values 
   */

  /* base ETSearchParams parameters */
  params->searchMaster = initSearchParams->nodeClass;
  params->rank         = 0;
  params->numSlaves    = 0;
  params->curSlaves    = 0;
  strcpy(params->chan1, initSearchParams->argv[5]);
  strcpy(params->chan2, initSearchParams->argv[6]);
  params->initialised  = 0;
  strcpy(params->id,"");

  /* initParams- initial search parameters */
  params->initParams->grbStart.gpsSeconds 
    = (INT4) atoi( initSearchParams->argv[1] );
  params->initParams->grbStart.gpsNanoSeconds 
    = (INT4) atoi( initSearchParams->argv[2] );
  params->initParams->sourceDir.longitude
    = (REAL8) atof( initSearchParams->argv[3] );
  params->initParams->sourceDir.latitude
    = (REAL8) atof( initSearchParams->argv[4] );
  params->initParams->sourceDir.system
    = COORDINATESYSTEM_EQUATORIAL;
  params->initParams->segLength
    = (UINT4) atoi( initSearchParams->argv[7] ) * GW_SAMPLE;
  params->initParams->numOffSegments
    = (UINT4) atoi( initSearchParams->argv[8] );
  params->initParams->onTimeShift
    = (INT4) ( atoi( initSearchParams->argv[9] ) +
	       atoi( initSearchParams->argv[7] )  );
  params->initParams->offTimeShiftLow
    = (UINT4) atoi( initSearchParams->argv[10] );
  params->initParams->offTimeShiftHigh
    = (UINT4) atoi( initSearchParams->argv[11] );
  params->initParams->initMask
    = (UINT4) atoi( initSearchParams->argv[12] );
#if DEBUG_EXTTRIG > 2
  fprintf( stdout, "grbStart = %d.%d \n",
	   params->initParams->grbStart.gpsSeconds,
	   params->initParams->grbStart.gpsNanoSeconds );
  fprintf( stdout, "sourceDir = lat: %6.4f,\tlong: %7.4f,\tsystem: %d \n",
	   params->initParams->sourceDir.latitude,
	   params->initParams->sourceDir.longitude,
	   params->initParams->sourceDir.system    );
  fprintf( stdout, "segLength = (in sec) %d\t (in counts) %d \n",
	   (params->initParams->segLength / GW_SAMPLE),
	   params->initParams->segLength               );
  fprintf( stdout, "numOffSegments = %d \n",
	   params->initParams->numOffSegments );
  fprintf( stdout, "onTimeShift  (in sec) = %d \n",
	   params->initParams->onTimeShift        );
  fprintf( stdout, "offTimeShift (in sec) = (low) %d,\t (high) %d \n",
	   params->initParams->offTimeShiftLow,
	   params->initParams->offTimeShiftHigh );
  fprintf( stdout, "initMask = %d \n", params->initParams->initMask );
  fflush( stdout );
#endif // #if DEBUG_EXTTRIG > 2

  /* result- output search results */
  params->result->variance1                = 0;
  params->result->variance2                = 0;
  params->result->xCorrUnit                = lalADCCountUnit;
  /* xCorrValue, xCorrDelay, and xCorrLag will be allocated on master node
   * in LALApplySearch() 
   */
  params->result->xCorrValue               = NULL;
  params->result->xCorrDelay               = NULL;
  params->result->xCorrLag                 = NULL;
  params->result->badSeconds               = 0;
  params->result->onSource                 = 0;

  /* create the ETSegList structure */
  params->segments = NULL;
  LALCreateETSegList( status->statusPtr, &(params->segments),
		      params->initParams);
  CHECKSTATUSPTR ( status );

  /* create the ETDataSegmentVector structure and RealFFTPlan structure */
  params->data = NULL;
  LALCreateETDataSegmentVector( status->statusPtr, &(params->data),
				params->initParams);
  CHECKSTATUSPTR ( status );

  params->fftPlan = NULL;
  /* the following computes a measured plan for (2x - 1) datapts --      */
  /* real time-series is zero-padded from -(N-1) to -1 so that resulting */
  /* complex frequency-series is the same length as optimal filter.      */
#if DEBUG_EXTTRIG > 2
  fprintf( stdout, "Creating ForwardRealFFTPlan, length %d \n",
	   (2 * params->initParams->segLength - 1)          );
  fflush( stdout );
#endif // DEBUG_EXTTRIG > 2
  LALCreateForwardRealFFTPlan( status->statusPtr, &(params->fftPlan),
			       (2 * params->initParams->segLength - 1), 1 );
  CHECKSTATUSPTR ( status );

  DETATCHSTATUSPTR (status);
  RETURN(status);
}

#ifdef __cplusplus
}
#endif
