/******** <lalVerbatim file="FCTHV"> ********
Author : Wen, L.
$ID$
********* </lalVerbatim> ********/

#ifndef _FCT_H
#define _FCT_H

#include <stdio.h> 
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <math.h>

#include <lal/LALStdlib.h> 
#include <lal/SeqFactories.h>
#include <lal/LALRCSID.h>
#include <lal/LALConstants.h>
#include "LALWrapperInterface.h"
#include <lal/LALStatusMacros.h>
#include <lal/LALFCTInterface.h>
#include "FCTMyPhaseFunc.h" /*  user supplied if #func>4 or if there is a name change */


/*  define this for verbose debugging messages  */

/*#define FCT_SO_DEBUG 1 */
#undef FCT_SO_DEBUG
/*#define FCT_SO_DEBUG_DATA  1 */
#undef FCT_SO_DEBUG_DATA   
/*#define FCT_SO_ATTACH_DDD  1 */
#undef FCT_SO_ATTACH_DDD 

#undef FCT_SO_GRAPHING_FUNCS


/* Physical constants within the FCT package */
#define FCTGENERALH_F0 (40.0L)
#define  Nyq (1024.0L)
#define  LambMax  (15.0) /* temporary */


#define  Ts 4.92549095e-6 /* GMs/c^3 */
#define  RsSun 2.9535e5  /* schwarzschild radius of the sun */ 
#define  Mpc 3.085678e24   
#define  powInd 1.1666666667 /*  f-dependence in amplitude */
#define  incl   0.0      /* in rad, fixed */


/* Number of events passed to subsequent hierarchical levels */
#define N_EVENTS (10)

/* Minimum power threshold */
#define MIN_THRESHOLD (0.0)


#ifdef  __cplusplus   /* C++ protection. */
extern "C" {
#endif


NRCSID( FCTGENERALH, "$ID$");

/******** <lalErrTable file="FCTHErrTab"> ********/
#define FCTGENERALH_ENULL 1
#define FCTGENERALH_EMEM  2
#define FCTGENERALH_MSGENULL "Null pointer."
#define FCTGENERALH_MSGEMEM  "Out of memory."
/******** </lalErrTable> ********/

typedef struct
tagFCTMomentType 
{
  COMPLEX8 avg;   
  COMPLEX8 sigma2;
}FCTMomentType;




typedef struct
tagFCTEventType
{
  LIGOTimeGPS                      time;
  REAL4                            effDist;
  REAL4                            mass1; 
  REAL4                            mass2;
  REAL4                            eta;
  REAL4                            t0; 
  REAL4                            t2; 
  REAL4                            t3; 
  REAL4                            t4; 
  REAL4                            tC; 
  REAL4                            snr;
  REAL4                            chisq;
  REAL8                            sigma;
  struct tagFCTEventType           *nextEvent;
}FCTEventType;

typedef struct 
tagFCTHierEventType
{ 
  REAL4                         *location; /* Array of size dimension */
  REAL4                         power;
  struct tagFCTHierEventType    *nextEvent;
}FCTHierEventType;


typedef struct
tagFCTHierLevelUnitsType{
  REAL4                         *range; /* physical range of each of the dimensions of the N-dimensional event */
  REAL4                         *physicalStepSize;
  INT4                          *indexRange;  /* range[i] / fctUnits[i] + 1 */
  REAL4                         freqCutOff;
  INT4                          parallelCutDimension;
  INT4                          reportCutDimension;
  UINT4                         numOfRowsInVol; /* May not need this. */
  REAL4                         *fctUnits;  /* lambda[i] = j * fctUnits[i]  after proper stride */
  INT4                          *stride;  
  
  /* We may need to use these.  phaseFunction(f) = phaseFunction(deltaFCTPhaseUnits * j + offsetFCTPhaseUnits) */
  /*  REAL4                         deltaFCTPhaseUnits;  
      REAL4                         offsetFCTPhaseUnits; */

  REAL4                         threshold;
  
}FCTHierLevelUnitsType;


  

typedef struct
tagFCTInParamType /*This structure is passed to LALMyFCTInterf LALFCTBuildEvent*/
{
  INT4                          dataLength;  /* After conditioning. Set in HierConditionData:ChangeFCTPlan */
  INT4                          dimension;    /* suggest: rename numDims */
  INT4                          numDataCubes; 
  INT4                          numEvents; /* assigned in FCTMyInterf.c, used anywhere? */
  LIGOTimeGPS                   epoch;  /* to be assigned in ConditionData */ 
  INT4                          maxF;   /* maximum frequency */ 
  REAL4                          dt;
  REAL4                          etaNorm; /* norm to calculate eta using t0 and t3, assigned in InitSearch */
  REAL4                          totalMassNorm; /* norm to calculate total mass using t0 and t3 */
  REAL8                          distNorm;  /* partly initialized in InitSearch, calculated in ConditionData */
  FCTMomentType                  *freqMoment; /* variance of h(f)/S(f), real and imaginary part*/
  REAL4                            prob;
  COMPLEX8Vector                *fctInput; /* frequency data after conditioning performed in HierConditionData. */ 
  LALFCTPlan                    *fctPlan;
  LALFCTDataCube                *cube;   /* datacube indicating where to perform the FCT. */
  INT4                           rowsPerApplySearch;  /* Used to figure out how often to report back. */
}FCTInParamType;  

/* local structures required  for LAL FCT  Interface initialization */

typedef struct
tagFCTInStrucType /* This structure is passed to LALMyFCTInitPlan */
{
  LALCreateFCTPlanInput  planIn;
  LALFCTSetUnitsInput    setUnitsIn;
  LALFCTPhaseFn          phaseFuncArray; /* Check pointer array to functions, global */
  INT4                   maxFFTSeg;
}FCTInStrucType;    


typedef struct
tagFCTSearchParams /* initialized in InitSearch() */
{
  /*REMOVE BOOLEAN                       searchMaster; */
  /*  BOOLEAN                       firstTime;*/ /*  need this? */
 
  UINT4                         *numProcs; /* pointer to a single int for MPI.  Not an Array. */
  
  FCTInParamType                *fctInParams;

  /*Hierarchy stuff */
  UINT4                         numHierLevels;  /* Hier = Hierarchy */
  UINT4                         curHierLevel;   /* The current Hier level.  Note that curHierLevel goes from 0 to numHierLevels - 1 */
  

  FCTHierLevelUnitsType         *hierLevelUnits; /* Array of FCTHierLevelUnitsType structures. */  

  FCTHierEventType              **hierLevelEvents; /* Array of pointers to the first event in each Hier level.*/
  INT4                          *numHierLevelEvents; /* int Array contianing the num of events found in each Hier level.*/
  INT4                          *lowestEvent;        /* event number of weakest event at each level */
  REAL4                         *powerThreshold;     /* array of power thresholds at each level */

  FCTHierEventType              *curEvent;  /* pointer to the current event that we're working on. */
  INT4                          curEventNum;  /* The number of the event in the linked list. Note that events 
						 are numbered from 0 to mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel] - 1 */
  BOOLEAN                       curEventConditioned; /* Has this Event been conditioned yet? */
  /*  UINT4                         curEventRowsDone;  */

  REAL4                         eventsPerNode; /* TotalNumber of events divided by the number of nodes. */

  REAL4                         eventStart;   /* floor(eventStart) indicates the start event integer.  eventStart - floor(eventStart) indicates
						 the fraction of the range of the parallel-cut dimension to start processing at. 
						 Note that there are two cut dimensions that are used to divide event processing.  One of these 
						 dimensions, the parallel-cut dimension is used to divide events onto different nodes.  The other,
						 the report-cut dimension, is used to report progress. 
					      */
  REAL4                         eventStop;   /* This indicates the end location of the events processed on this node.  See above comments. */
  
  INT4                          numReportSlices; /* This is the number of report slices in the current working fraction of an event on the current node. */ 
  INT4                          curReportSlice;  /* A number indicating which report slice we are working on.  Range: 0 to numReportSlices - 1 */
  
  
  /* Some information about the input data before we do anything. */
  INT4                          inputDataLength; 
  REAL4                         inputDataStart; /* Start time (seconds) */
  REAL4                         inputDataStop; /* Stop time (seconds) */
  REAL4                         inputDataStepSize; /* sampling time in seconds */
  LIGOTimeGPS                   inputDataStartGPS; 
    
  /* The mass range (in solar masses) that we are doing the search over. */
  REAL4                         minMass; 
  REAL4                         maxMass;
}FCTSearchParams;

void
LALBuildFCTOutput (
		   LALStatus          *status,
		   LALSearchOutput    *output,
		   INT4               numEvents,
		   FCTEventType       *event
		   );



void
LALMyFCTInitPlan(LALStatus *status, FCTInStrucType *fctInStrucs, LALFCTPlan  **fctPlan);

void 
LALMyFCTInterf(LALStatus *status,  FCTHierEventType **event, FCTSearchParams *mySearchParams);

/*
void
LALFCTInitEvent(LALStatus *status, FCTEventType **event);
*/

void
LALFCTBuildEvent(LALStatus *status, COMPLEX8Vector *fctOut, FCTSearchParams *mySearchParams);

void
LALFCTMoment(LALStatus *status, INT4 length, COMPLEX8 *dataIn, FCTMomentType *moment);

#include <FCTHierSupport.h>

  /* table explanation */

  /*
  IFO	  CHARACTER	2	819		N		
  SEARCH	VARCHAR	24	819		N		
  END_TIME	INTEGER	4	0		N		
  END_TIME_NS	INTEGER	4	0		N		
  IMPULSE_TIME	INTEGER	4	0		Y		
  IMPULSE_TIME_NS INTEGER	4	0	Y		
  AMPLITUDE	REAL	4	0		N		
  EFF_DISTANCE	REAL	4	0		Y		
  COA_PHASE	REAL	4	0		Y		
  MASS1	REAL	4	0		Y		
  MASS2	REAL	4	0		Y		
  MCHIRP	REAL	4	0		Y		
  ETA	REAL	4	0		Y		
  TAU0	REAL	4	0		Y		
  TAU2	REAL	4	0		Y		
  TAU3	REAL	4	0		Y		
  TAU4	REAL	4	0		Y		
  TAU5	REAL	4	0		Y		
  TTOTAL	REAL	4	0		Y		
  SNR	REAL	4	0		Y		
  CHISQ	REAL	4	0		Y		
  CHISQ_DOF	INTEGER	4	0		Y		
  SIGMASQ	DOUBLE	8	0		Y		
  */




#ifdef  __cplusplus
}
#endif  /* C++ protection. */


#endif /* _FCT_H */










