/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeLALApplySearchC"> ********
Author: Daw, E
$Id: SlopeLALApplySearch.c,v 1.1 2002/05/08 14:26:34 edaw Exp $
********* </lalVerbatim> ********/

#ifdef DEBUG_SLOPE
#include <stdio.h>
#endif
#include <math.h>

/* functions for memory allocation */
#include <lal/LALStdlib.h>

/* functions for vector creation and manipulation */
/* #include <lal/AVFactories.h> */

/* functions for LAL MPI interface */
#include <lal/Comm.h>

/* functions for slope filtering and peak searching data */
#include <lal/SlopeDetectorFilter.h>

/* structures and definitions for this wrapper code */
#include <SlopeDatatypes.h>
#include <SlopeApplySearchMaster.h>
#include <SlopeMakeEventTable.h>

NRCSID( SLOPELALAPPLYSEARCHC, "$Id: SlopeLALApplySearch.c,v 1.1 2002/05/08 14:26:34 edaw Exp $");

/******** local static function declarations ********/

/****************************************************/
/** note that currently my code is designed to run **/
/** on a single machine.                           **/
/****************************************************/

/******** wrapperAPI function definitions */

/* apply the slope filter to the data using the slopefilters */
/* lal package. Apply a peak search algorithm to the filtered */
/* data and add an entry to the event linked list for every */
/* peak that passes the cut. Increment the output data field */
/* that keeps track of the percentage of the analysis completed. */

void
LALApplySearch(LALStatus*                status,
	       LALSearchOutput*          output,
               LALSearchInput*           input,
	       LALApplySearchParams*     applyParams)
{
  SlopeSearchParams* params;

  INITSTATUS( status, "SlopeLALApplySearch", SLOPELALAPPLYSEARCHC );
  ATTATCHSTATUSPTR(status);

  /* extract my parameters from the search parameters */
  params = (SlopeSearchParams*)applyParams->searchParams ;

  /* figure out your rank */
  MPI_Comm_rank(*(applyParams->mpiParams->comm) , &(params->mpiRank) );

  /* if you are the master, start looking for peaks. Otherwise */
  /* abort with an error because I currently don't support multiprocessing */
  if(params->mpiMaster) {
#ifdef SLDEBUG
    printf("Applying search master routine because I am the master\n");
#endif
    SlopeApplySearchMaster(status->statusPtr,output,params);
  } else {
    ABORT( status, APPLYSEARCH_NOSLAVES, APPLYSEARCH_NOSLAVESMESG );
  }

  /* write the linked list of located events to the */
  /* database */
#ifdef SLDEBUG
  printf("making event table.\n");
#endif

  if(params->numberOfTriggersSoFar > 0) {
    SlopeMakeEventTable(status->statusPtr, params, output);
  }

#ifdef SLDEBUG
  printf("made event table.\n");
#endif

  /* set done bit on output data */
  output->notFinished = 0;

  CHECKSTATUSPTR(status);
  DETATCHSTATUSPTR(status);
  RETURN(status);
}
